library(CSTools)
library(lubridate)
library(startR)
library(s2dv)
source("R/Interpolation.R")

target_grid <- '/esarchive/recon/ecmwf/era5/monthly_mean/tas_f1h/tas_200002.nc'

lonmin <- -11.5
lonmax <- 5.35
latmin <- 35.1
latmax <- 44.1
extra <- 1

sdates <- format(ymd("20000201") + rep(years(0:8), each=1),"%Y%m%d")
#---------------------------
# Model
#---------------------------
# ecmwf- system5c3s : coarse resolution
exp <- startR::Start(dat = '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1), member = 'all', sdate = sdates,
                     lat = values(list(latmin - extra, latmax + extra)), lat_reorder = Sort(decreasing = FALSE), 
                     lon = values(list(lonmin - extra, lonmax + extra)), lon_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'), 
                                     lat = c('lat', 'latitude'), member = c('member','ensemble')),
                     return_vars = list(lat = 'dat', lon = 'dat'),
                     num_procs = 1, retrieve = TRUE)

exps2dv <- as.s2dv_cube(exp)


#--------------------------------
# Downscaling with Interpolation
#--------------------------------
Int_con <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat,
                          lons = attr(exp, "Variables")$dat1$lon, target_grid = target_grid, method_remap = 'con')
Int_bil <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat, 
                          lons = attr(exp, "Variables")$dat1$lon, target_grid = target_grid, method_remap = 'bil')
Int_bic <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat,
                          lons = attr(exp, "Variables")$dat1$lon, target_grid = target_grid, method_remap = 'bic')
Int_nn <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat,
                         lons = attr(exp, "Variables")$dat1$lon, target_grid = target_grid, method_remap = 'nn')

#--------------------------------
# Downscaling with CST_Interpolation
#--------------------------------
cstInt_con <- CST_Interpolation(exp = exps2dv, target_grid = target_grid, method_remap = 'con')
cstInt_bil <- CST_Interpolation(exp = exps2dv, target_grid = target_grid, method_remap = 'bil')
cstInt_bic <- CST_Interpolation(exp = exps2dv, target_grid = target_grid, method_remap = 'bic')
cstInt_nn <- CST_Interpolation(exp = exps2dv, target_grid = target_grid, method_remap = 'nn')

#--------------------------------
# Downscaling to point location
#--------------------------------

points <- list(lat = c(36.1, 38.7), lon = c(-1.9, 0.8))
Int_points_1 <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat, lons = attr(exp, "Variables")$dat1$lon,
                             points = points, method_point_interp = 'bilinear', source_file = attr(exp, "Files")[1,1,1])

Int_points_2 <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat, lons = attr(exp, "Variables")$dat1$lon,
                             points = points, method_point_interp = 'nearest', source_file = attr(exp, "Files")[1,1,1])

Int_points_3 <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat, lons = attr(exp, "Variables")$dat1$lon,
                             points = points, method_point_interp = '9point', source_file = attr(exp, "Files")[1,1,1])

Int_points_4 <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat, lons = attr(exp, "Variables")$dat1$lon,
                             points = points, method_point_interp = 'invdist4nn', source_file = attr(exp, "Files")[1,1,1])

Int_points_5 <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat, lons = attr(exp, "Variables")$dat1$lon,
                             points = points, method_point_interp = 'NE', source_file = attr(exp, "Files")[1,1,1])

Int_points_6 <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat, lons = attr(exp, "Variables")$dat1$lon,
                             points = points, method_point_interp = 'NW', source_file = attr(exp, "Files")[1,1,1])

Int_points_7 <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat, lons = attr(exp, "Variables")$dat1$lon,
                             points = points, method_point_interp = 'SE', source_file = attr(exp, "Files")[1,1,1])

Int_points_8 <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat, lons = attr(exp, "Variables")$dat1$lon,
                             points = points, method_point_interp = 'SW', source_file = attr(exp, "Files")[1,1,1])

cstInt_points_1 <- CST_Interpolation(exp = exps2dv, points = points, method_point_interp = 'bilinear')
cstInt_points_2 <- CST_Interpolation(exp = exps2dv, points = points, method_point_interp = 'nearest')
cstInt_points_3 <- CST_Interpolation(exp = exps2dv, points = points, method_point_interp = '9point')
cstInt_points_4 <- CST_Interpolation(exp = exps2dv, points = points, method_point_interp = 'invdist4nn')
cstInt_points_5 <- CST_Interpolation(exp = exps2dv, points = points, method_point_interp = 'NE')
cstInt_points_6 <- CST_Interpolation(exp = exps2dv, points = points, method_point_interp = 'NW')
cstInt_points_7 <- CST_Interpolation(exp = exps2dv, points = points, method_point_interp = 'SE')
cstInt_points_8 <- CST_Interpolation(exp = exps2dv, points = points, method_point_interp = 'SW')
#--------------------------------
# Downscaling to specific region
#--------------------------------
crplonmin <- -1
crplonmax <- 4 
crplatmin <- 39
crplatmax <- 44

region_ext = c(crplonmin, crplonmax, crplatmin, crplatmax)

Int_points <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat, lons = attr(exp, "Variables")$dat1$lon,
                             method_remap = 'bil',  target_grid = target_grid, region = region_ext, source_file = attr(exp, "Files")[1,1,1])

cstInt_points <- CST_Interpolation(exp = exps2dv, method_remap = 'bil',  target_grid = target_grid, region = region_ext)

#------------------------------------
# Downscaling for daily data
#------------------------------------
#---------------------------
# Model
#---------------------------
# ecmwf- system5c3s : coarse resolution
exp <- startR::Start(dat = '/esarchive/exp/ecmwf/system5c3s/daily_mean/$var$_f6h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1:28), member = indices(1:3), sdate = format(ymd(sdates), "%Y%m%d"),
                     latitude = values(list(latmin - extra , latmax + extra)), latitude_reorder = Sort(decreasing = FALSE), 
                     longitude = values(list(lonmin - extra, lonmax + extra)), longitude_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'), 
                                     latitude = c('lat', 'latitude'), member = c('member','ensemble')),
                     return_vars = list(latitude = 'dat', longitude = 'dat'),
                     num_procs = 1, retrieve = TRUE)


exps2dv <- as.s2dv_cube(exp)

#------------------------------------
# Downscaling with different methods
#------------------------------------

Int_daily <- Interpolation(exp = exp, lats = attr(exp, "Variables")$dat1$lat,
                           lons = attr(exp, "Variables")$dat1$lon, target_grid = target_grid, method_remap = 'con',
                           lon_dim = 'longitude', lat_dim = 'latitude')

cstInt_daily <- CST_Interpolation(exp = exps2dv, target_grid = target_grid, method_remap = 'con',
                                  lon_dim = 'longitude', lat_dim = 'latitude')

