### interpolation for lr


# predictor maybe means obsL analog 

library(startR)
library(s2dv)
library(CSTools)
library(lubridate)
library(multiApply)
source('R/Interpolation.R')
source('R/Utils.R')
source('R/Intlr.R')

target_grid <- '/esarchive/recon/ecmwf/era5/monthly_mean/tas_f1h/tas_200002.nc'

lonmin <- -11.5
lonmax <- 5.35
latmin <- 35.1
latmax <- 44.1
extra <- 1

sdates <- format(ymd("20000201") + rep(years(0:8), each=1),"%Y%m%d")
sdates_fcst <- format(seq(ymd("20230201"), ymd("20230401"), "1 month"), "%Y%m%d")

#---------------------------
# Observations
#---------------------------
# ERA5-Land observations: high-resolution observations
obs <- startR::Start(dat = '/esarchive/recon/ecmwf/era5land/monthly_mean/$var$_f1h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1), lat = values(list(latmin - extra, latmax + extra)),
                     lat_reorder = Sort(decreasing = FALSE), lon = values(list(lonmin - extra, lonmax + extra)),
                     lon_reorder = CircularSort(-180, 180), sdate = format(ymd(sdates), "%Y%m"), 
                     synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'), 
                                     lat = c('lat', 'latitude')), return_vars = list(lat = 'dat', lon = 'dat'),
                     num_procs = 1, retrieve = TRUE)

obs_era5 <- startR::Start(dat = '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h/$var$_$sdate$.nc',
                          var = 'tas', time = indices(1), latitude = values(list(latmin - extra, latmax + extra)), 
                          sdate = format(ymd(sdates), "%Y%m"), latitude_reorder = Sort(decreasing = FALSE),
                          longitude = values(list(lonmin - extra, lonmax + extra)), 
                          longitude_reorder = CircularSort(-180, 180),
                          synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'), 
                                          latitude = c('lat', 'latitude')), return_vars = list(latitude = 'dat', longitude = 'dat'),
                          num_procs = 1, retrieve = TRUE)

#---------------------------
# Model
#---------------------------
# ecmwf- system5c3s : coarse resolution
exp <- startR::Start(dat = '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1), member = 'all', sdate = sdates,
                     lat = values(list(latmin - extra, latmax + extra)), lat_reorder = Sort(decreasing = FALSE), 
                     lon = values(list(lonmin - extra, lonmax + extra)), lon_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'), 
                                     lat = c('lat', 'latitude'), member = c('member','ensemble')),
                     return_vars = list(lat = 'dat', lon = 'dat'),
                     num_procs = 1, retrieve = TRUE)

#---------------------------
# Forecast
#---------------------------
# ecmwf- system5c3s : coarse resolution
fcst <- startR::Start(dat = '/esarchive/exp/ecmwf/system51c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
                      var = 'tas', time = indices(1), member = 'all', sdate = sdates_fcst,
                      lat = values(list(latmin - extra, latmax + extra)), lat_reorder = Sort(decreasing = FALSE),
                      lon = values(list(lonmin - extra, lonmax + extra)), lon_reorder = CircularSort(-180, 180),
                      synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'),
                                      lat = c('lat', 'latitude'), member = c('member','ensemble')),
                      return_vars = list(lat = 'dat', lon = 'dat'),
                      num_procs = 1, retrieve = TRUE)

obss2dv <- as.s2dv_cube(obs)
exps2dv <- as.s2dv_cube(exp)
fcsts2dv <- as.s2dv_cube(fcst)


#----------------------------------
# Downscaling with Intlr
#----------------------------------


down_1 <- Intlr(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat, 
                obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, lr_method = "basic",
                int_method = "bilinear", predictors = NULL, loocv = TRUE, ncores = 4)

down_2 <- Intlr(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat,
                exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, lr_method = "large-scale",
                int_method = "nn", predictors = NULL, loocv = FALSE, ncores = 4)

down_3 <- Intlr(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat,
                exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, lr_method = "9nn",
                int_method = "cone", predictors = NULL, loocv = TRUE, ncores = 4)

cstdown_1 <- CST_Intlr(exp = exps2dv, obs = obss2dv, target_grid = target_grid, lr_method = "basic",
                       int_method = "bic", predictors = NULL, loocv = TRUE, ncores = 4)
#----------------------------------
# Downscaling with predictor
#----------------------------------
down_prdct <- Intlr(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat, 
                    exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat, 
                    obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, lr_method = "large-scale",
                    int_method = "bic", predictors = obs_era5, loocv = TRUE, ncores = 4,
                    large_scale_predictor_dimname = 'var')

cstdown_prdct <- CST_Intlr(exp = exps2dv, obs = obss2dv, target_grid = target_grid, lr_method = "large-scale",
                           int_method = "bil", predictors = obs_era5, 
                           large_scale_predictor_dimname = 'var',
                           loocv = FALSE, ncores = 4) #taking so much time............

#----------------------------------
# Downscaling to point locations
#----------------------------------
points <- list(lat = c(36.1, 38.7), lon = c(-1.9, 0.8))

down_points_1 <- Intlr(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat,
                       exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                       obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, lr_method = "basic",
                       points = points, method_point_interp = 'invdist4nn', int_method = 'con', predictors = NULL, 
                       source_file_exp = attr(exp, "Files")[1,1,1], source_file_obs = attr(obs, "Files")[1,1,1],
                       loocv = TRUE, ncores = 4)

down_points_2 <- Intlr(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat,
                       exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                       obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, lr_method = "large-scale",
                       points = points, method_point_interp = 'SE', int_method = 'con', predictors = obs_era5,
                       large_scale_predictor_dimname = 'var',
                       source_file_exp = attr(exp, "Files")[1,1,1], source_file_obs = attr(obs, "Files")[1,1,1],
                       loocv = TRUE, ncores = 4)

cstdown_points_1 <- CST_Intlr(exp = exps2dv, obs = obss2dv, target_grid = target_grid, lr_method = "basic",
                              points = points, method_point_interp = 'SE',
                              int_method = "bilinear", predictors = NULL, loocv = TRUE, ncores = 4)


#----------------------------------
# Downscaling to point locations given point observations
#----------------------------------
points <- list(lat = c(36.1, 38.7), lon = c(-1.9, 0.8))
obs_point <- Subset(obs, along = c("lat", "lon"), indices = list(1:2,1), drop = FALSE)
obs_point <- MergeDims(data = obs_point, merge_dims = c("lat", "lon"), rename_dim = 'location')

down_obspoint <- Intlr(exp = exp, obs = obs_point, exp_lats = attr(exp, "Variables")$dat1$lat,
                       exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = points$lat, obs_lons = points$lon, 
                       target_grid = target_grid, lr_method = "basic", points = points, 
                       method_point_interp = 'bilinear', int_method = 'bilinear', predictors = NULL, 
                       source_file_exp = attr(exp, "Files")[1,1,1], source_file_obs = attr(obs, "Files")[1,1,1], 
                       loocv = TRUE, ncores = 1)

cstdown_obspoints <- CST_Intlr(exp = exps2dv, obs = obss2dv, target_grid = target_grid, lr_method = "basic",
                               points = points, method_point_interp = 'bilinear',
                               int_method = "bilinear", predictors = NULL, loocv = TRUE, ncores = 4)

#-----------------------------------------------
# Downscaling to specified region
#-----------------------------------------------
crplonmin <- -1
crplonmax <- 4 
crplatmin <- 39
crplatmax <- 44

region_ext = c(crplonmin, crplonmax, crplatmin, crplatmax)

down_region_1 <- Intlr(exp = exp, obs = obs, exp_cor = fcst, exp_lats = attr(exp, "Variables")$dat1$lat,
                       exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                       obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, lr_method = "basic",
                       int_method = "bilinear", predictors = NULL, loocv = TRUE, ncores = 4, region = region_ext)


down_region_2 <- Intlr(exp = exp, obs = obs,  exp_lats = attr(exp, "Variables")$dat1$lat,
                       exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                       obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, lr_method = "basic",
                       int_method = "bilinear",  loocv = TRUE, ncores = 4, region = region_ext,
                       predictors = obs_era5, large_scale_predictor_dimname = 'var')

cstdown_region_1 <- CST_Intlr(exp = exps2dv, obs = obss2dv, target_grid = target_grid, lr_method = "basic", region = region_ext,
                              int_method = "bilinear", predictors = NULL, loocv = TRUE, ncores = 4)

cstdown_region_2 <- CST_Intlr(exp = exps2dv, obs = obss2dv, target_grid = target_grid, lr_method = "basic",
                              region = region_ext, predictors = obs_era5, large_scale_predictor_dimname = 'var',
                              int_method = "bilinear", loocv = TRUE, ncores = 4)

#------------------------------------
# Downscaling forecasts using hindcasts to train
#------------------------------------

down_fcst_1 <- Intlr(exp = exp, obs = obs, exp_cor = fcst, exp_lats = attr(exp, "Variables")$dat1$lat,
                     exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                     obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, lr_method = "basic",
                     int_method = "bilinear", predictors = NULL, loocv = TRUE, ncores = 4)


down_fcst_2 <- Intlr(exp = exp, obs = obs, exp_cor = fcst, exp_lats = attr(exp, "Variables")$dat1$lat,
                     exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                     obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, lr_method = "large-scale",
                     int_method = "bilinear", predictors = obs_era5, loocv = FALSE, ncores = 4,
                     large_scale_predictor_dimname = 'var') ### not working fcst and obsera5 together have some problem


cstdown_fcst_1 <- CST_Intlr(exp = exps2dv, obs = obss2dv, exp_cor = fcsts2dv, target_grid = target_grid, 
                            lr_method = "basic", points = points, method_point_interp = 'bilinear',
                            int_method = "bilinear", predictors = NULL, loocv = TRUE, ncores = 4)

cstdown_fcst_1 <- CST_Intlr(exp = exps2dv, obs = obss2dv, exp_cor = fcsts2dv, target_grid = target_grid, 
                            lr_method = "basic", points = points, method_point_interp = 'bilinear',
                            int_method = "bilinear", predictors = NULL, loocv = TRUE, ncores = 4,
                            predictors = obs_era5, large_scale_predictor_dimname = 'var') # not working


#------------------------------------
# Downscaling for daily data
#------------------------------------

#---------------------------
# Observations
#---------------------------
# ERA5-Land observations: high-resolution observations
lonmin <- -11.5
lonmax <- 5.35
latmin <- 35.1
latmax <- 44.1
extra <- 1

obs <- startR::Start(dat = '/esarchive/recon/ecmwf/era5land/daily_mean/$var$_f1h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1:28), latitude = values(list(latmin - extra, latmax + extra)),
                     sdate = format(ymd(sdates), "%Y%m"), latitude_reorder = Sort(decreasing = FALSE),
                     longitude = values(list(lonmin - extra, lonmax + extra)),
                     longitude_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'),
                                     latitude = c('lat', 'latitude')), return_vars = list(latitude = 'dat', longitude = 'dat'),
                     num_procs = 1, retrieve = TRUE)

#---------------------------
# Model
#---------------------------
# ecmwf- system5c3s : coarse resolution
exp <- startR::Start(dat = '/esarchive/exp/ecmwf/system5c3s/daily_mean/$var$_f6h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1:28), member = indices(1:3), sdate = format(ymd(sdates), "%Y%m%d"),
                     latitude = values(list(latmin - extra , latmax + extra)), latitude_reorder = Sort(decreasing = FALSE), 
                     longitude = values(list(lonmin - extra, lonmax + extra)), longitude_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), longitude = c('lon', 'longitude'), 
                                     latitude = c('lat', 'latitude'), member = c('member','ensemble')),
                     return_vars = list(latitude = 'dat', longitude = 'dat'),
                     num_procs = 1, retrieve = TRUE)


obss2dv <- as.s2dv_cube(obs)
exps2dv <- as.s2dv_cube(exp)

#------------------------------------
# Downscaling with different methods
#------------------------------------

down_daily <- Intlr(exp = exp, obs = obs, exp_lats = attr(exp, "Variables")$dat1$lat,
                    exp_lons = attr(exp, "Variables")$dat1$lon, obs_lats = attr(obs, "Variables")$dat1$lat,
                    obs_lons = attr(obs, "Variables")$dat1$lon, target_grid = target_grid, lr_method = "basic",
                    int_method = 'con', predictors = NULL, 
                    source_file_exp = attr(exp, "Files")[1,1,1], source_file_obs = attr(obs, "Files")[1,1,1],
                    loocv = TRUE, ncores = 4, lon_dim='longitude', lat_dim='latitude')


# Parameter 'indices' has different length of parameter data in the dimension supplied in 'split_dim'.problem in last lines after if(daily)
cstdown_daily <- CST_Intlr(exp = exps2dv, obs = obss2dv, target_grid = target_grid, lr_method = "basic",
                       lon_dim='longitude', lat_dim='latitude',
                       int_method = "bic", predictors = NULL, loocv = TRUE, ncores = 4)



