source('R/Interpolation.R')
source('R/Utils.R')
source('R/Intlr.R')
require(testthat)
require(CSTools)
require(multiApply)
require(s2dv)

set.seed(1)

exp <- rnorm(500)
dim(exp) <- c(member = 5, lat = 4, lon = 5, sdate = 5, time  = 1)
exp_lons <- 1:5
exp_lats <- 1:4
exps2dv <- suppressWarnings(s2dv_cube(data = exp, lat = exp_lats, lon = exp_lons)) 

obs <- rnorm(900)
dim(obs) <- c(lat = 12, lon = 15, sdate = 5, time = 1)
obs_lons <- seq(1,5, 4/14)
obs_lats <- seq(1,4, 3/11)
obss2dv <- suppressWarnings(s2dv_cube(data = obs, lat = obs_lats, lon = obs_lons))

exp_cor <- exp


exp1 <- exp
names(dim(exp1)) <- c("member", "lats", "lons", "sdate", "time")

exp_cor1 <- exp1
exp_cor2 <- exp1
names(dim(exp_cor2)) <- c("member", "lats", "lons", "sdates", "time")

exp_cor3 <- exp1
dim(exp_cor3) <- c(var = 1, member = 5, lat = 4, lon = 5, sdate = 5, time  = 1)
exp_cor4 <- exp1
dim(exp_cor4) <- c(var = 2, member = 5, lat = 2, lon = 5, sdate = 5, time  = 1)

obs2 <- obs
dim(obs2) <- c(lat = 6, lon = 15, sdate = 5, time = 1, member = 2)

test_that("1. Input checks", {
  
  expect_error(
    CST_Intlr(exp = 1),
    "Parameter 'exp' must be of the class 's2dv_cube'")
  
  expect_error(
    CST_Intlr(exp = exps2dv, obs = 1),
    "Parameter 'obs' must be of the class 's2dv_cube'")
  
  expect_error(
    CST_Intlr(exp = exps2dv, obs = obss2dv, exp_cor = 1, lr_method = 'basic'),
    "Parameter 'exp_cor' must be of the class 's2dv_cube'")
  
})

test_that("2. Input checks", {
  
  expect_error(
    Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 1, int_method = 'conservative'),
    "Parameter 'lr_method' must be of the class 'character'")
  
  expect_error(
    Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', int_method = 'conservative',
          large_scale_predictor_dimname = 2),
    "Parameter 'large_scale_predictor_dimname' must be of the class 'character'")
  
  expect_error(
    Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', int_method = 'conservative',
          loocv = 2),
    "Parameter 'loocv' must be set to TRUE or FALSE")
  
  expect_error(
    Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', int_method = 'conservative',
          lat_dim = 2),
    "Parameter 'lat_dim' must be of the class 'character'")
  
  expect_error(
    Intlr(exp = exp1, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', int_method = 'conservative'),
    "Missing longitude dimension in 'exp', or does not match the parameter 'lon_dim'")
  
  expect_error(
    Intlr(exp = exp, obs = obs, exp_cor = exp_cor2,
          exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', int_method = 'conservative'),
    "Missing start date dimension in 'exp_cor', or does not match the parameter 'sdate_dim'")
  
  expect_error(
    suppressWarnings(
      Intlr(exp = exp, obs = obs, exp_cor = exp_cor1,
            exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
            obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', int_method = 'conservative')),
    "Missing longitude dimension in 'exp_cor', or does not match the parameter 'lon_dim'")
  
  expect_warning(
    Intlr(exp = exp, obs = obs, exp_cor = exp_cor,
          exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', int_method = 'conservative',
          loocv = T),
    paste0("Forecast data will be downscaled. ",
           "'loocv' is set to FALSE to train the model with the whole hindcast and predict with the forecast."))
  
  expect_warning(
    Intlr(exp = exp, obs = obs, exp_cor = exp_cor,
          exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = '9nn', int_method = 'conservative',
          loocv = F),
    paste0("9nn method skips the interpolation step since the method itself inherently downscale ",
           "the coarse resolution data to the reference data resolution without the need ",
           "for interpolation."))
  
  expect_error(
    Intlr(exp = exp, obs = obs, points = list(lats = c(1,2), lons = c(3,4)),
          exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', 
          int_method = 'conservative', source_file_exp = 'null.nc'),
    paste0("Please provide the interpolation method to interpolate gridded data to ",
           "point locations through the parameter 'method_point_interp'."))
  
  expect_error(
    Intlr(exp = exp, obs = obs2, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', 
          int_method = 'conservative'),
    paste0("Not implemented for observations with members ('obs' can have 'member_dim', ",
           "but it should be of length = 1)."),
    fixed = TRUE)
  
  expect_error(
    Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', 
          int_method = 'conservative', predictors = TRUE),
    paste0("Parameter 'predictors' must be of the class 'array'"))
  
  expect_error(
    Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', 
          int_method = 'conservative', predictors = TRUE),
    paste0("Parameter 'predictors' must be of the class 'array'"))
  
  expect_error(
    suppressWarnings(
      Intlr(exp = exp, obs = obs, exp_cor = exp_cor4,
            exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
            obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', 
            int_method = 'conservative', predictors = exp_cor3,
            large_scale_predictor_dimname = 'var')),
    paste0("Large scale predictor dimension in exp_cor and predictors must be identical."))
  
  expect_error(
    Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', 
          int_method = 'conservative', ncores = -1),
    paste0("Parameter 'ncores' must be a positive integer."))
  
  expect_error(
    Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic'),
    paste0("Parameter 'int_method' must be a character vector indicating the interpolation method. ",
           "Accepted methods are con, bil, bic, nn, con2"))
  
  expect_warning(
    Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', int_method = 'bil'),
    paste0("The borders of the downscaling region have not been provided. Assuming the ",
           "four borders of the downscaling region are defined by the first and last ",
           "elements of the parametres 'obs_lats' and 'obs_lons'."))
  
  expect_error(
    suppressWarnings(
      Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
            obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'large-scale', int_method = 'bil')),
    "The large-scale predictors must be passed through the parametre 'predictors'")
  
  expect_error(
    suppressWarnings(
      Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
            obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'nn', int_method = 'bil')),
    "nn method is not implemented yet")
  
  expect_warning(
    Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = '9nn', int_method = 'bil'),
    "Interpolating to the 'obs' grid")
})

test_that("3. Output checks", {
  res1 <- suppressWarnings(
    CST_Intlr(exp = exps2dv, obs = obss2dv, lr_method = "basic", 
              int_method = "bil", target_grid = 'r1280x640'))
  
  expect_equal(class(res1), "list")
  expect_equal(length(res1), 2)
  expect_equal(names(res1), c("exp", "obs"))
  expect_equal(names(res1$exp), names(res1$obs))
  expect_equal(names(res1$exp), names(res1$obs))
  expect_equal(dim(res1$exp$data)[["lat"]], 39)
  expect_equal(dim(res1$exp$data)[["lon"]], 50)
  expect_equal(dim(res1$exp$data)[["member"]], dim(exps2dv$data)[["member"]])
  
  expect_equal(length(res1$obs$coords[["lon"]]), 50)
  expect_equal(names(dim(res1$obs$data)), names(dim(obss2dv$data)))
  
  expect_equal(round(res1$exp$data[1:3],3), c(0.168, 0.082, 0.222))
  
}) 

test_that("4. Output checks", {
  res2 <- suppressWarnings(
    Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', int_method = 'conservative'))
  
  expect_equal(class(res2), "list")
  expect_equal(names(res2), c("data", "obs", "lon", "lat"))
  expect_equal(dim(res2$data),c(time = 1, sdate = 5, member = 5, lat = 10, lon = 14))
  expect_equal(round(res2$data[1,1,1,1,1:3], 3), c(-0.918, -0.436, -0.766))
  
  res3 <- suppressWarnings(
    Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = '9nn', int_method = 'conservative'))
  
  expect_equal(round(res3$data[1,1,1,1,1:3], 3), c(0.385, -1.033, -0.802))
  
  res4 <- suppressWarnings(
    Intlr(exp = exp, obs = obs, exp_lats = exp_lats, exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, target_grid = 'r1280x640', lr_method = 'basic', int_method = 'bil'))
  
  expect_equal(round(res4$data[1,1,1,1,1:3], 3), c(-2.005, -0.533,  0.130))
  
})



