set.seed(1)
source('R/Interpolation.R')
source('R/Utils.R')
source('R/Intbc.R')
require(testthat)
require(CSTools)
require(multiApply)
require(s2dv)

exp <- rnorm(500)
dim(exp) <- c(member = 5, lat = 4, lon = 5, sdate = 5, time  = 1)
exp_lons <- 1:5
exp_lats <- 1:4

coords <- list(member = 5, lat = exp_lats, lon = exp_lons, sdate = 5, time = 1)
exps2dv <- list(data = exp, coords = coords)
attr(exps2dv, 'class') <- 's2dv_cube'

obs <- rnorm(900)
dim(obs) <- c(lat = 12, lon = 15, sdate = 5, time = 1)
obs_lons <- seq(1,5, 4/14)
obs_lats <- seq(1,4, 3/11)

coords <- list(member = 5, lat = obs_lats, lon = obs_lons, sdate = 5, time = 1)
obss2dv <- list(data = obs, coords = coords)
attr(obss2dv, 'class') <- 's2dv_cube'

exp_cor <- rnorm(500)
dim(exp_cor) <- c(member = 5, lat = 4, lon = 5, sdate = 5, time  = 1)
exp_cor_lons <- 1:5
exp_cor_lats <- 1:4

coords <- list(member = 5, lat = exp_cor_lats, lon = exp_cor_lons, sdate = 5, time = 1)
exp_cors2dv <- list(data = exp_cor, coords = coords)
attr(exp_cors2dv, 'class') <- 's2dv_cube'

exp1 <- exp
names(dim(exp1)) <- c("member", "latin", "lonin", "sdate", "time")

exp_cor1 <- exp_cor
names(dim(exp_cor1)) <- c("member", "latin", "lonin", "sdate", "time")


test_that("1. Input checks: CST_Intbc", {
  expect_error(
    CST_Intbc(exp = 1),
    "Parameter 'exp' must be of the class 's2dv_cube'")
  
  expect_error(
    CST_Intbc(exp = exps2dv, obs = obs, obsL = 1),
    paste0("Parameter 'obs' must be of the class 's2dv_cube'")
  )
  
})

test_that("2. Input checks: Intbc", {
  
  expect_error(
    Intbc(exp = exp, obs = obs, exp_lats = exp_lats, 
          exp_lons = exp_lons, obs_lats = obs_lats,
          obs_lons = obs_lons, target_grid = 'r1280x640', 
          bc_method = 1, int_method = 'con'),
    "Parameter 'bc_method' must be of the class 'character'"
  )
  
  expect_error(
    Intbc(exp = exp, obs = obs, exp_lats = exp_lats, 
          exp_lons = exp_lons, obs_lats = obs_lats, lat_dim = c(1:2),
          obs_lons = obs_lons, target_grid = 'r1280x640', 
          bc_method = 'bias', int_method = 'con'),
    "Parameter 'lat_dim' must be of the class 'character'"
  )
  
  expect_error(
    Intbc(exp = exp1, obs = obs, exp_lats = exp_lats, 
          exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, 
          bc_method = 'bias',target_grid = 'r1280x640'),
    "Missing longitude dimension in 'exp', or does not match the parameter 'lon_dim'"
  )
  
  expect_error(
    Intbc(exp = exp, obs = obs, exp_lats = exp_lats, 
          exp_lons = exp_lons, obs_lats = obs_lats, 
          obs_lons = obs_lons, 
          bc_method = 'simple_bias',target_grid = 'r1280x640'),
    paste0("Parameter 'bc_method' must be a character vector indicating the bias adjustment method. ",
           "Accepted methods are 'quantile_mapping', 'dynamical_bias', 'bias', 'evmos', 'mse_min', 'crps_min', 'rpc-based'. ",
           "The abbreviations 'dbc','qm' can also be used."
    ))
  
  expect_error(
    Intbc(exp = exp, obs = obs, exp_cor = exp_cor1,
          exp_lats = exp_lats, exp_lons = exp_lons, 
          obs_lats = obs_lats, obs_lons = obs_lons, 
          lon_dim = "lon", lat_dim = "lat", int_method = "con",
          bc_method = 'bias',target_grid = 'r1280x640'),
    "Missing longitude dimension in 'exp_cor', or does not match the parameter 'lon_dim'"
  )
  
  expect_error(
    Intbc(exp = exp, obs = obs, exp_cor = exp_cor, 
          exp_lats = exp_lats, exp_lons = exp_lons, 
          obs_lats = obs_lats, obs_lons = obs_lons, 
          points = list(lat = 1, lon =2),
          lon_dim = "lon", lat_dim = "lat", int_method = "con",
          bc_method = 'bias',target_grid = 'r1280x640'),
    "No source file found. Source file must be provided in the parameter 'source_file_exp'."
  )
  
  expect_error(
    Intbc(exp = exp, obs = obs, exp_cor = exp_cor, 
          exp_lats = exp_lats, exp_lons = exp_lons, 
          obs_lats = obs_lats, obs_lons = obs_lons, 
          points = list(lat = 1, lon =2), source_file_exp = 'nulll.nc',
          lon_dim = "lon", lat_dim = "lat", int_method = "con",
          bc_method = 'bias',target_grid = 'r1280x640'),
    paste0("Please provide the interpolation method to interpolate gridded data to point ",
           "locations through the parameter 'method_point_interp'.")
  )
  
  expect_warning(
    Intbc(exp = exp, obs = obs, exp_cor = exp_cor, 
          exp_lats = exp_lats, exp_lons = exp_lons, 
          obs_lats = obs_lats, obs_lons = obs_lons, 
          lon_dim = "lon", lat_dim = "lat", int_method = "con",
          bc_method = 'bias',target_grid = 'r1280x640'),
    paste0("The borders of the downscaling region have not been provided. ",
           "Assuming the four borders of the downscaling region are defined by the first ",
           "and last elements of the parametres 'obs_lats' and 'obs_lons'.")
  )
  
  expect_warning(
    Intbc(exp = exp, obs = obs, exp_cor = exp_cor, 
          exp_lats = exp_lats, exp_lons = exp_lons, 
          obs_lats = obs_lats, obs_lons = obs_lons, 
          lon_dim = "lon", lat_dim = "lat", int_method = "con",
          bc_method = 'bias',target_grid = 'r1280x640'),
    paste0("The borders of the downscaling region have not been provided. ",
           "Assuming the four borders of the downscaling region are defined by the first ",
           "and last elements of the parametres 'obs_lats' and 'obs_lons'.")
  )
  
  expect_error(
    suppressWarnings(
      Intbc(exp = exp, obs = obs,  
            exp_lats = exp_lats, exp_lons = exp_lons, 
            obs_lats = obs_lats, obs_lons = obs_lons, 
            lon_dim = "lon", lat_dim = "lat", 
            bc_method = 'bias',target_grid = 'r1280x640', ncores = -1)),
    paste0("Parameter 'ncores' must be a positive integer.")
  )
  
  expect_warning(
    Intbc(exp = exp, obs = obs, exp_cor = exp_cor, 
          exp_lats = exp_lats, exp_lons = exp_lons, 
          obs_lats = obs_lats, obs_lons = obs_lons, 
          lon_dim = "lon", lat_dim = "lat", int_method = 'con',
          bc_method = 'bias',target_grid = 'r1280x640', loocv = T),
    paste0("Forecast data will be downscaled. 'loocv' is set to FALSE to train ",
           "the model with the whole hindcast and predict with the forecast.")
  )
  
})

test_that("3. Output checks: CST_Intbc", {
  
  res <- suppressWarnings(
    CST_Intbc(exp = exps2dv, obs = obss2dv, 
              target_grid = 'r1280x640', bc_method = 'bias',
              int_method = 'con'))
  
  expect_equal(names(res), c("exp", "obs"))
  expect_equal(names(res$exp), c("data", "coords", "dims"))
  expect_equal(names(res$obs), c("data", "coords", "dims"))
  expect_equal(dim(res$obs$data), c(lat = 10, lon = 14, sdate = 5, time = 1))
  expect_equal(dim(res$exp$data), c(member = 5, lat = 10, lon = 14, sdate = 5, time = 1))
  
  expect_equal(round(res$exp$data[1,1,1,,1], 3), c(-0.500, -0.421, 0.428, 0.816, 1.027))
  expect_equal(round(res$obs$data[1,1,,1], 3), c(-0.156, -0.975, 0.356, 1.149, 0.720))
  
})

test_that("4. Output checks: Intbc", {
  
  res1 <- suppressWarnings(
    Intbc(exp = exp, obs = obs, exp_lats = exp_lats, 
          exp_lons = exp_lons, obs_lats = obs_lats,
          obs_lons = obs_lons, target_grid = 'r1280x640', 
          bc_method = 'bias', int_method = 'con'))
  
  expect_equal(dim(res1$data), c(member = 5, lat = 10, lon = 14, sdate = 5, time = 1))
  expect_equal(round(res1$data[1,1,1,,1], 3), c(-0.500, -0.421, 0.428, 0.816, 1.027))
  expect_equal(round(res1$obs[1,1,,1], 3), c(-0.156, -0.975, 0.356, 1.149, 0.720))
  
})
