# test for rscripts

source('R/Interpolation.R')
source('R/Utils.R')
source('R/Analogs.R')
require(testthat)
require(CSTools)
require(multiApply)
require(s2dv)

set.seed(1)

######################
obs <- rnorm(900)
time = 1:5
latitude <- seq(43, 45, by = 0.5)
longitude <- seq(-2.5, 3, by = 0.5)
sdate <- format(ymd("20000501") + rep(years(0:2), each=1),"%Y%m")
dim(obs) <- c(data = 1, var = 1, sdate = 3, lon = 12, lat = 5, time = 5)

# coords <- list(data = obs,  var = "tas", lat = latitude, lon = longitude, sdate = sdate, time = time)
# obss2dv <- s2dv_cube(data = obs, coords = coords)

coords <- list(data = 1,  var = "tas", lat = latitude, lon = longitude, sdate = sdate, time = time)
obs <- list(data = obs, coords = coords)
obss2dv <- obs
attr(obss2dv, 'class') <- 's2dv_cube'

######################
obsL <- rnorm(270)
time = 1:5
latitude <- seq(43, 45, by = 1)
longitude <- seq(-2.5, 3, by = 1)
sdate <- format(ymd("20000501") + rep(years(0:2), each=1),"%Y%m")
dim(obsL) <- c(data = 1, var = 1, sdate = 3, lon = 6, lat = 3, time = 5)

#coords <- list(data = obsL,  var = "tas", lat = latitude, lon = longitude, sdate = sdate, time = time )
#obsLs2dv <- s2dv_cube(data = obsL, coords = coords)

coords <- list(data = 1,  var = "tas", lat = latitude, lon = longitude, sdate = sdate, time = time)
obsL <- list(data = obsL, coords = coords)
obsLs2dv <- obsL
attr(obsLs2dv, 'class') <- 's2dv_cube'

######################
exp <- rnorm(315)
time = 1:5
latitude <- seq(43, 45, by = 0.9)
longitude <- seq(-2.5, 3, by = 0.9)
sdate <- format(ymd("20000501") + rep(years(0:2), each=1),"%Y%m")
dim(exp) <- c(data = 1, var = 1, sdate = 3, lon = 7, lat = 3, time = 5)

# coords <- list(data = exp,  var = "tas", lat = latitude, lon = longitude, sdate = sdate, time = time )
# exps2dv <- s2dv_cube(data = exp, coords = coords)

coords <- list(data = 1,  var = "tas", lat = latitude, lon = longitude, sdate = sdate, time = time)
exp <- list(data = exp, coords = coords)
exps2dv <- exp
attr(exps2dv, 'class') <- 's2dv_cube'

# Analogs(exp = exp$data, obs = obs$data, grid_exp = "r360x180", 
#         lat_dim = "lat", lon_dim = "lon",
#         exp_lats = exp$coords$lat,  exp_lons = exp$coords$lon, 
#         obs_lats = obs$coords$lat, obs_lons = obs$coords$lon)



obs1 <- obs
dim(obs1$data) <- c(data = 1, var = 1, sdate = 3, lon = 12, lat = 5, time = 1,  member = 5)
obs2 <- obs
dim(obs2$data) <- c(data = 1, var = 1, sdate = 5, lon = 12, lat = 5, time = 3)
obs3 <- obs
dim(obs3$data) <- c(data = 1, var = 1, sdate = 3, lon = 12, lat = 5, time = 5, window = 1)

exp1 <- exp
names(dim(exp1$data)) <- c("data", "var", "sdate", "lons", "lats", "time")
exp2 <- exp
names(dim(exp2$data)) <- c("data", "var", "sdates", "lon", "lat", "times")

obsL1 <- obs
names(dim(obsL1$data)) <- c("data", "var", "sdate", "lons", "lats", "time")
obsL2 <- obs
dim(obsL2$data) <- c(data = 1, var = 1, sdate = 3, lon = 12, lat = 5, time = 1, window = 5)



## test_that
test_that("1. Input checks: CST_Analogs", {
  # Check 's2dv_cube'
  expect_error(
    CST_Analogs(exp = 1, obs = 1),
    paste0("Parameter 'obs' must be of the class 's2dv_cube'")
  )
  
  expect_error(
    CST_Analogs(exp = 1, obs = obss2dv, obsL = 1),
    paste0("Parameter 'obsL' must be of the class 's2dv_cube'")
  )
  
  expect_error(
    CST_Analogs(exp = 1, obs = obss2dv, obsL = obsLs2dv),
    paste0("Parameter 'exp' must be of the class 's2dv_cube'")
  )
})


test_that("2. Input checks: Analogs", {

  expect_error(
    Analogs(exp = exp$data, obs = obs$data, grid_exp = 1),
    paste0("Parameter 'grid_exp' must be of class 'character'. ",
           "It can be either a path to another NetCDF file which to read the target grid from ",
           "(a single grid must be defined in such file) or a character vector indicating ",
           "the coarse grid to be passed to CDO, and it must be a grid recognised by CDO or a NetCDF file."),
    fixed = TRUE
  )

  expect_error(
    Analogs(exp = exp$data, obs = obs$data, grid_exp = "r360x180", nanalogs = "c"),
    "Parameter 'nanalogs' must be of the class 'numeric'"
  )
  
  expect_error(
    Analogs(exp = exp1$data, obs = obs$data, grid_exp = "r360x180", 
            exp_lats = exp1$coords$lat,  exp_lons = exp1$coords$lon, 
            obs_lats = obs$coords$lat, obs_lons = obs$coords$lon
            ),
    "Missing longitude dimension in 'exp' and/or 'obs', or does not match the parameter 'lon_dim'"
  )
  
  expect_error(
    Analogs(exp = exp$data, obs = obs1$data, grid_exp = "r360x180",
            lat_dim = "lat", lon_dim = "lon",
            exp_lats = exp$coords$lat,  exp_lons = exp$coords$lon,
            obs_lats = obs1$coords$lat, obs_lons = obs1$coords$lon
    ),
    "Not implemented for observations with members ('obs' can have 'member_dim', but it should be of length = 1).",
    fixed = TRUE
  )

  expect_error(
    Analogs(exp = exp2$data, obs = obs$data, grid_exp = "r360x180", 
            lat_dim = "lat", lon_dim = "lon",
            exp_lats = exp2$coords$lat, exp_lons = exp2$coords$lon, 
            obs_lats = obs$coords$lat, obs_lons = obs$coords$lon
    ),
    "Missing start date dimension in 'exp' and/or 'obs', or does not match the parameter 'sdate_dim'"
  )
   
  expect_error(
    Analogs(exp = exp$data, obs = obs$data, obsL = obsL1$data,
            grid_exp = "r360x180", 
            exp_lats = exp$coords$lat, exp_lons = exp$coords$lon, 
            obs_lats = obs$coords$lat, obs_lons = obs$coords$lon,
            obsL_lats = obsL1$coords$lat, obsL_lons = obsL1$coords$lon
    ),
    "Missing longitude dimension in 'obsL', or does not match the parameter 'lon_dim'"
  )

  expect_error(
    Analogs(exp = exp$data, obs = obs$data, grid_exp = "r360x180", ncores = -2.4),
    "Parameter 'ncores' must be a positive integer."
  )
  
  expect_warning(
    Analogs(exp = exp$data, obs = obs2$data, grid_exp = "r360x180", 
            lat_dim = "lat", lon_dim = "lon",
            exp_lats = exp$coords$lat, exp_lons = exp$coords$lon, 
            obs_lats = obs2$coords$lat, obs_lons = obs2$coords$lon, loocv_window = TRUE),
    paste0("The lengths of sdate_dim in the data provided through exp and obs are not the same. ", 
    "Thus, exp is considered as forecast data, and loocv_window is set to FALSE.")
  )

  expect_error(
    suppressWarnings(
    Analogs(exp = exp$data, obs = obs3$data, obsL = obs$data,
            grid_exp = "r360x180", lat_dim = "lat", lon_dim = "lon",
            exp_lats = exp$coords$lat, exp_lons = exp$coords$lon, 
            obs_lats = obs3$coords$lat, obs_lons = obs3$coords$lon, 
            obsL_lats = obs$coords$lat, obsL_lons = obs$coords$lon,
            loocv_window = TRUE)),
    paste0("Either both obs and obsL should include 'window' dimension or none.")
  )
  
  expect_error(
    suppressWarnings(
      Analogs(exp = exp$data, obs = obs$data, obsL = obsL2$data,
            grid_exp = "r360x180", lat_dim = "lat", lon_dim = "lon",
            exp_lats = exp$coords$lat, exp_lons = exp$coords$lon, 
            obs_lats = obs$coords$lat, obs_lons = obs$coords$lon, 
            obsL_lats = obsL2$coords$lat, obsL_lons = obsL2$coords$lon,
            loocv_window = TRUE)),
    paste0("Either both obs and obsL should include 'window' dimension or none")
  )
  
  expect_warning(
    Analogs(exp = exp$data, obs = obs$data, grid_exp = "r360x180", 
            lat_dim = "lat", lon_dim = "lon",
            exp_lats = exp$coords$lat, exp_lons = exp$coords$lon, 
            obs_lats = obs$coords$lat, obs_lons = obs$coords$lon, loocv_window = TRUE),
    paste0("The borders of the downscaling region have not been provided. ",
           "Assuming the four borders of the downscaling region are defined by the first ",
           "and last elements of the parameters 'exp_lats' and 'exp_lons'.")
  )
})  


test_that("3. Output checks: Analogs", {
  suppressWarnings(
    res <- Analogs(exp = exp$data, obs = obs$data, grid_exp = "r360x180", 
                   lat_dim = "lat", lon_dim = "lon",
                   exp_lats = exp$coords$lat,  exp_lons = exp$coords$lon, 
                   obs_lats = obs$coords$lat, obs_lons = obs$coords$lon)
  )
  
  expect_equal(
    names(res),
    c("data", "obs", "lon", "lat")
  )
  
  expect_equal(
    dim(res$data),
    c(lat = 5, lon = 12, analogs = 3, sdate = 3, data = 1, var = 1, time = 5)
  )
  
  smry <- c(-3.00804860, -0.70397648, -0.07450517, -0.03345535,  0.66712164,  3.81027668)
  expect_equal(
    as.numeric(summary(res$data)),
    smry
  )
  
  suppressWarnings(
    res1 <- Analogs(exp = exp$data, obs = obs$data, obsL = obsL$data,
                    grid_exp = "r360x180", 
                    exp_lats = exp$coords$lat, exp_lons = exp$coords$lon, 
                    obs_lats = obs$coords$lat, obs_lons = obs$coords$lon,
                    obsL_lats = obsL$coords$lat, obsL_lons = obsL$coords$lon)
  )
  
  expect_equal(
    as.numeric(summary(res$data)),
    c(-3.00804860, -0.70397648, -0.07450517, -0.03345535,  0.66712164,  3.81027668)
  )
  suppressWarnings(
    res2 <- Analogs(exp = exp$data, obs = obs$data, grid_exp = "r360x180", 
                    lat_dim = "lat", lon_dim = "lon",
                    exp_lats = exp$coords$lat, exp_lons = exp$coords$lon,
                    obs_lats = obs$coords$lat, obs_lons = obs$coords$lon, 
                    loocv_window = TRUE, return_indices = T)
  )
  
  expect_equal(
    names(res2),
    c("data", "ind", "obs", "lon", "lat")
  )
  
  crplonmin <- -1
  crplonmax <- 2
  crplatmin <- 43.5
  crplatmax <- 44.5
  
  region_ext = c(crplonmin, crplonmax, crplatmin, crplatmax)
  
  ana_region <- suppressWarnings(
    Analogs(exp = exp$data, obs = obs$data, 
            grid_exp = "r360x180", 
            exp_lats = exp$coords$lat, exp_lons = exp$coords$lon, 
            obs_lats = obs$coords$lat, obs_lons = obs$coords$lon,
            region = region_ext,
            return_indices = TRUE, ncores = 4))
  
  expect_equal(dim(ana_region$data)[["lat"]], 3)
  expect_equal(length(ana_region$lat), 5)
  
  expect_equal(dim(ana_region$data)[["lon"]], 7)
  expect_equal(length(ana_region$lon), 12 )
  
})
  

test_that("4. Output checks: CST_Analogs", {
  
  suppressWarnings(
  res3 <-  CST_Analogs(exp = exps2dv, obs = obss2dv, obsL = obsLs2dv, grid_exp = "r360x180")
  )
  
  expect_equal(
    class(res3),
    "list"
  )
  
  expect_equal(
    names(res3),
    c("exp", "obs")
  )
  
  expect_equal(
    names(res3$exp),
    c("data", "coords", "dims")
  )
  
  expect_equal(
    names(res3$obs),
    c("data", "coords", "dims")
  )
  
  expect_equal(
    names(res3$obs$coords),
    c("data", "var", "lat", "lon", "sdate", "time")
  )
  
  expect_equal(
    dim(res3$obs$data),
    c(data = 1, var = 1, sdate = 3, lon = 12, lat = 5, time = 5)
  )
  
  expect_equal(
    dim(res3$exp$data),
    c(lat = 5, lon = 12, analogs =3, sdate = 3,  data = 1, var = 1, time = 5)
  )
  
  expect_equal(
    as.numeric(summary(res3$exp$data)),
    c(-3.00804860, -0.68126366, -0.06884093, -0.04161696,  0.64224131,  3.81027668)
  )
  
  expect_equal(
    as.numeric(summary(res3$obs$data)),
    c(-3.00804860,-0.69095134, -0.04633189, -0.01339405,  0.68851708,  3.81027668)
  )
  
  suppressWarnings(
  res4 <- CST_Analogs(exp = exps2dv, obs = obss2dv, obsL = obsLs2dv, 
                      grid_exp = "r360x180", fun_analog = "mean")
  )
  
  suppressWarnings(
  res5 <- Analogs(exp = exp$data, obs = obs$data, obsL = obsL$data,
                  grid_exp = "r360x180", fun_analog = "mean",
                  exp_lats = exp$coords$lat, exp_lons = exp$coords$lon, 
                  obs_lats = obs$coords$lat, obs_lons = obs$coords$lon,
                  obsL_lats = obsL$coords$lat, obsL_lons = obsL$coords$lon)
  )
  
  expect_equal(
    as.numeric(summary(res4$exp$data)),
    c(-1.43983315, -0.44040967, -0.05595320, -0.04161696,  0.32877385,  1.98308257)
  )
  
  expect_equal(
    summary(res4$exp$data),
    summary(res5$data)
    )
  
}) 



